C                      PORGRAMME LOVE.FOR
C LOVE.FOR computes the `effective' internal Love numbers for the
C Shell (Mantle plus Crust) and Inner Core for an input Earth model.
C These Love numbers allow the radial displacement at the boundaries of the 
C Outer fluid Core to be expressed in terms of the generalized
C displacement potential there.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION R(100),RHO(100),GZERO(100),RI(300),RHOI(300),
     1GZEROI(300),ENAME(10),NM(4),NI(4),NK(4),B(98,198),C(100,100),
     2Y(6,6),YSCAL(6,3),YSCAL2(6,3),YA(6),A(6,6),AS(6,6),CAUG(6,13),
     3YP(6,6),YIC(6,3),Y1(6,6),Y2(6,6),DELTA(6,3),CM1(5,5),CM2(5,5),
     4CM3(5,5),BV1(5),BV2(5),BV3(5),CAUGC(3,7),CMC1(3,3),CMC2(3,3),
     5CMC3(3,3),BVC1(3),BVC2(3),BVC3(3),CAUGS(5,11),CAUGS0(4,9),
     6CM01(4,4),CM02(4,4),CM03(4,4),BV01(4),BV02(4),BV03(4)
      DOUBLE PRECISION MU(100),LAMBDA(100),MUI(300),LAMBDAI(300),
     1K1(6,6),K2(6,6),K3(6,6),K4(6,6),LAMBDAC,LAMBDBM
      CHARACTER*20 EMODEL
C Enter Earth model file name.
      WRITE(6,10)
 10   FORMAT(1x,'Type in Earth model file name.')
      READ(5,11)EMODEL
 11   FORMAT(A20)
C Open Earth model file.
      OPEN(UNIT=1,FILE=EMODEL,STATUS='OLD')
C Open Love number file.
      OPEN(UNIT=2,FILE='love.dat',STATUS='UNKNOWN')
C Set angular frequency of Earth's rotation.
      WE=7.2921151467D-5
C Calculate square of angular frequency of Earth's rotation.
      WES=WE*WE
C Set value of PI.
      PI=3.141592653589793D0
C Set value of Universal Constant of Gravitation.
      G=6.6732D-11
C Enter period, azimuthal number, inertial and Coriolis switches.
      WRITE(6,12)
 12   FORMAT(1X,'Enter period, azimuthal number and inertial and Corioli
     1s switches (1 for in, 0 for out)')
      READ(5,*)TPER,MAZ,INERT,ICOR
C Convert period to seconds.
      TPER=3600.D0*TPER
C Calculate dimensionless angular frequency.
      SIG=PI/(TPER*WE)
C Calculate square of dimensionless angular frequency.
      SIGS=SIG*SIG
C Compute square of angular frequency.
      ANGS=DFLOAT(INERT)
      ANGS=2.D0*PI*ANGS/TPER
      ANGS=ANGS*ANGS
C Compute Coriolis term.
      COR=2.D0*DFLOAT(MAZ)*DFLOAT(ICOR)
      COR=2.D0*PI*COR*WE/TPER
C Set maximum relative error tolerance for integrations.
      EPS=1.D-5
C Set minimum starting radius for variable Earth properties.
      RMIN=1200.D1
C Set maximum dimensions for interpolation.
      M1=100
      M2=198
      M3=98
C Read in and write out Earth model.
C Read in and write out headers.
C Earth model name.
      READ(1,13)(ENAME(I),I=1,10)
 13   FORMAT(10A7)
      WRITE(2,13)(ENAME(I),I=1,10)
      WRITE(6,14)(ENAME(I),I=1,10)
 14   FORMAT(11X,10A7)
C Number of Love numbers to be computed.
      READ(1,15)NN
 15   FORMAT(I10)
      WRITE(2,15)NN
C Number of model points and number of integration steps.
      READ(1,16)(NM(I),NI(I),I=1,4)
 16   FORMAT(8I10)
      WRITE(6,17)
 17   FORMAT(/,/,16X,'Number of model points and number of integration s
     1teps.',/)
      WRITE(6,16)(NM(I),NI(I),I=1,4)
      WRITE(6,18)
 18   FORMAT(/,/,5X,'Radius',6X,'Rho',4X,'Lambda',7X,'Mu',6X,'Gzero')
      WRITE(6,19)
 19   FORMAT(6X,'(km)',5X,'(gm/cc)',2X,'(kbars)',4X,'(kbars)',2X,'(cm/se
     1c/sec)',/)
C Read in Earth model, Inner Core, Outer Core, Mantle and Crust.
      K=0
      DO 20 M=1,4
        N1=NM(M)
        READ(1,21)(R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I),I=1,N1)
 21     FORMAT(1X,F10.1,F10.2,F10.1,F10.1,F10.1)
        DO 22 I=1,N1
C Scale Earth model to SI values and store.
          J=K+I
          RI(J)=R(I)*1.D3
          RHOI(J)=RHO(I)*1.D3
          LAMBDAI(J)=LAMBDA(I)*1.D8
          MUI(J)=MU(I)*1.D8
          GZEROI(J)=GZERO(I)*1.D-2
C Write Earth model out to screen.
          WRITE(6,21)R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I)
 22     CONTINUE
           NK(M)=K
           K=K+N1
 20   CONTINUE
C Find density, lambda above, gravity at, the Inner Core-Outer Core
C boundary.
      RHOAC=RHOI(NK(2)+1)
      LAMBDAC=LAMBDAI(NK(2)+1)
      GZEROC=GZEROI(NK(2)+1)
C Find density, lambda below, gravity at, the Core-Mantle boundary.
      RHOBM=RHOI(NK(3))
      LAMBDBM=LAMBDAI(NK(3))
      GZEROM=GZEROI(NK(3))
C Store radii of Inner Core-Outer Core and Core-Mantle boundaries
C and surface radius.
      RIOB=RI(NK(2))
      RCMB=RI(NK(3))
      D=RI(NK(4)+NM(4))
C Write out radii of ICB and CMB.
      WRITE(2,23) RIOB,RCMB
C Write out gravity at the ICB and the CMB.
      WRITE(2,23) GZEROC,GZEROM
 23   FORMAT(2D23.15)
C Begin calculation of Love numbers for the Inner Core.
C Set up interpolation for the Inner Core.
      N1=NM(1)
      N2=2*N1-2
      N3=N1-2
C Put Inner Core values in active locations.
      DO 24 I=1,N1
        J=NK(1)+I
        R(I)=RI(J)
        RHO(I)=RHOI(J)
        MU(I)=MUI(J)
        LAMBDA(I)=LAMBDAI(J)
        GZERO(I)=GZEROI(J)
 24   CONTINUE
C Calculate gravity gradient at geocentre.
      GAMMA=(4.D0*PI*G*RHO(1)-2.D0*WES)/3.D0
C Set minimum radius for variable Earth properties.
      R(1)=RMIN
C Set gravity at minimum radius.
      GZERO(1)=GAMMA*RMIN
C Construct interpolation matrix.
      CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Begin loop over degree for Inner Core.
      WRITE(6,25)
 25   FORMAT(1X,/,/,11X,'Degree',14X,'Love number h1',14X,'Love number h
     12')
      DO 26 NP1=1,NN
        N=NP1-1
        AN=DFLOAT(N)
C Test if N=0.
        IF(N.EQ.0) GO TO 27
C Test if N=1 and set counter.
        IF(N.NE.1) GO TO 28
        ICOUNT=1
C Store ANGS and COR
        ANGSS=ANGS
        CORS=COR
C Set ANGS and COR to zero for calculation of DC value.
        ANGS=0.D0
        COR=0.D0
C Begin power series expansions of fundamental solutions for N>0.
C Set initial values of fundamental solutions.
C Set initial values of first fundamental solution.
 28     CONTINUE
        Y(1,1)=1.D0
        Y(2,1)=2.D0*(AN-1.D0)*MU(1)
        Y(3,1)=1.D0/AN
        Y(4,1)=Y(2,1)/AN
        Y(5,1)=4.D0*PI*G*RHO(1)/AN
        Y(6,1)=0.D0
C Set initial values of second fundamental solution.
        Y(1,2)=0.D0
        Y(2,2)=0.D0
        Y(3,2)=0.D0
        Y(4,2)=0.D0
        Y(5,2)=1.D0/AN
        Y(6,2)=1.D0
C Set initial values of third fundamental solution.
        Y(3,3)=P2(AN,LAMBDA(1),MU(1))/P1(AN,LAMBDA(1),MU(1))
        Y(2,3)=-Q1(AN,LAMBDA(1),MU(1))*Y(3,3)+Q2(AN,LAMBDA(1),MU(1))
        Y(1,3)=-AN*Y(3,3)+1.D0/MU(1)
        Y(4,3)=1.D0
        Y(5,3)=2.D0*PI*G*RHO(1)*((AN+3.D0)*Y(1,3)-AN*(AN+1.D0)*Y(3,3))/
     1  (2.D0*AN+3.D0)
        Y(6,3)=(AN+2.D0)*Y(5,3)-4.D0*PI*G*RHO(1)*Y(1,3)
C Calculate coefficients of second terms in series expansions.
C Calculate coefficients of second terms for first fundamental solution.
        YSCAL(3,1)=RHO(1)*((3.D0-AN)*GAMMA+ANGS+2.D0*WES-COR/AN)/
     1  P1(AN,LAMBDA(1),MU(1))
        YSCAL(2,1)=-Q1(AN,LAMBDA(1),MU(1))*YSCAL(3,1)
        YSCAL(1,1)=-AN*YSCAL(3,1)
        YSCAL(4,1)=0.D0
        YSCAL(5,1)=2.D0*PI*G*RHO(1)*((AN+3.D0)*YSCAL(1,1)
     1  -AN*(AN+1.D0)*YSCAL(3,1))/(2.D0*AN+3.D0)
        YSCAL(6,1)=(AN+2.D0)*YSCAL(5,1)-4.D0*PI*G*RHO(1)*YSCAL(1,1)
C Calculate coefficients of second terms for second fundamental solution.
        YSCAL(3,2)=RHO(1)/P1(AN,LAMBDA(1),MU(1))
        YSCAL(2,2)=-Q1(AN,LAMBDA(1),MU(1))*YSCAL(3,2)
        YSCAL(1,2)=-AN*YSCAL(3,2)
        YSCAL(4,2)=0.D0
        YSCAL(5,2)=2.D0*PI*G*RHO(1)*((AN+3.D0)*YSCAL(1,2)
     1  -AN*(AN+1.D0)*YSCAL(3,2))/(2.D0*AN+3.D0)
        YSCAL(6,2)=(AN+2.D0)*YSCAL(5,2)-4.D0*PI*G*RHO(1)*YSCAL(1,2)
C Generate matrix for evaluation of second terms in power series
C expansion of third fundamental solution.
        CALL MATRIX(A,RHO,LAMBDA,MU,PI,G,AN,WE)
C Set right hand side for calculation of second terms in third solution.
        YA(1)=0.D0
        YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*Y(1,3)+
     1  (AN*(AN+1.D0)*GAMMA+COR)*Y(3,3)-Y(6,3))
        YA(3)=0.D0
        YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*Y(1,3)-
     1  (ANGS-COR/(AN*(AN+1.D0)))*Y(3,3)-Y(5,3))
        YA(5)=0.D0
        YA(6)=0.D0
C Store matrix A in AS.
        DO 29 I=1,6
          DO 30 J=1,6
            AS(I,J)=A(I,J)
 30       CONTINUE
 29    CONTINUE  
C Solve for second terms in third solution.
        CALL LINSOL(A,YA,6,CAUG,DET,6,13)
        YSCAL(1,3)=YA(1)
        YSCAL(2,3)=YA(2)
        YSCAL(3,3)=YA(3)
        YSCAL(4,3)=YA(4)
        YSCAL(5,3)=YA(5)
        YSCAL(6,3)=YA(6)
C Set right hand side for calculation of third terms in first solution.
        YA(1)=0.D0
        YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,1)+
     1  (AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,1)-YSCAL(6,1))
        YA(3)=0.D0
        YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,1)-
     1  (ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,1)-YSCAL(5,1))
        YA(5)=0.D0
        YA(6)=0.D0
C Copy stored matrix AS into matrix A.
        DO 31 I=1,6
          DO 32 J=1,6
            A(I,J)=AS(I,J)
 32       CONTINUE
 31     CONTINUE
C Solve for third terms in first solution.
        CALL LINSOL(A,YA,6,CAUG,DET,6,13)
        YSCAL2(1,1)=YA(1)
        YSCAL2(2,1)=YA(2)
        YSCAL2(3,1)=YA(3)
        YSCAL2(4,1)=YA(4)
        YSCAL2(5,1)=YA(5)
        YSCAL2(6,1)=YA(6)
C Set right hand side for calculation of third terms in second solution.
        YA(1)=0.D0
        YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,2)+
     1  (AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,2)-YSCAL(6,2))
        YA(3)=0.D0
        YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,2)-
     1  (ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,2)-YSCAL(5,2))
        YA(5)=0.D0
        YA(6)=0.D0
C Copy stored matrix AS into matrix A.
        DO 33 I=1,6
          DO 34 J=1,6
            A(I,J)=AS(I,J)
 34       CONTINUE
 33     CONTINUE
C Solve for third terms in second solution.
        CALL LINSOL(A,YA,6,CAUG,DET,6,13)
        YSCAL2(1,2)=YA(1)
        YSCAL2(2,2)=YA(2)
        YSCAL2(3,2)=YA(3)
        YSCAL2(4,2)=YA(4)
        YSCAL2(5,2)=YA(5)
        YSCAL2(6,2)=YA(6)
C Set right hand side for calculation of third terms in third solution.
        YA(1)=0.D0
        YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,3)+
     1  (AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,3)-YSCAL(6,3))
        YA(3)=0.D0
        YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,3)-
     1  (ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,3)-YSCAL(5,3))
        YA(5)=0.D0
        YA(6)=0.D0
C Copy stored matrix AS into matrix A and add 2.0 to diagonal of A.
        DO 35 I=1,6
          DO 36 J=1,6
            A(I,J)=AS(I,J)
            IF(I.EQ.J)A(I,J)=A(I,J)+2.D0
 36       CONTINUE
 35     CONTINUE
C Solve for third terms in third solution.
        CALL LINSOL(A,YA,6,CAUG,DET,6,13)
        YSCAL2(1,3)=YA(1)
        YSCAL2(2,3)=YA(2)
        YSCAL2(3,3)=YA(3)
        YSCAL2(4,3)=YA(4)
        YSCAL2(5,3)=YA(5)
        YSCAL2(6,3)=YA(6)
        GO TO 37 
 27     CONTINUE
C Begin power series expansion of fundamental solution for degree n=0.
C Set first two solutions to zero.
        DO 38 I=1,6
          DO 39 J=1,2
            Y(I,J)=0.D0
            YSCAL(I,J)=0.D0
            YSCAL2(I,J)=0.D0
 39       CONTINUE  
 38     CONTINUE
C Set values for first three terms of fundamental solution.
        Y(1,3)=1.D0
        Y(2,3)=3.D0*LAMBDA(1)+2.D0*MU(1)
        Y(3,3)=0.D0
        Y(4,3)=0.D0
        Y(5,3)=2.D0*PI*G*RHO(1)
        Y(6,3)=0.D0
        YSCAL(1,3)=-RHO(1)*(4.D0*GAMMA+ANGS+2.D0*WES)/
     1  (10.D0*(LAMBDA(1)+2.D0*MU(1)))
        YSCAL(2,3)=(5.D0*LAMBDA(1)+6.D0*MU(1))*YSCAL(1,3)
        YSCAL(3,3)=0.D0
        YSCAL(4,3)=0.D0
        YSCAL(5,3)=PI*G*RHO(1)*YSCAL(1,3)
        YSCAL(6,3)=0.D0
        YSCAL2(1,3)=-RHO(1)*(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,3)/
     1  (4.D0*(7.D0*LAMBDA(1)+14.D0*MU(1)))
        YSCAL2(2,3)=(7.D0*LAMBDA(1)+10.D0*MU(1))*YSCAL2(1,3)
        YSCAL2(3,3)=0.D0
        YSCAL2(4,3)=0.D0
        YSCAL2(5,3)=2.D0*PI*G*RHO(1)*YSCAL2(1,3)/3.D0
        YSCAL2(6,3)=0.D0
 37     CONTINUE
C Set last three columns of array Y(6,6) to zero for Inner Core.
        DO 40 I=1,6
          DO 41 J=4,6
            Y(I,J)=0.D0
 41       CONTINUE
 40     CONTINUE    
C Put Inner Core solutions in array YIC(6,3).
        DO 42 I=1,6
          DO 43 J=1,3
            YIC(I,J)=Y(I,J)
 43       CONTINUE  
 42     CONTINUE
C Begin Runge-Kutta integration for the Inner Core.
C Find maximum ratio of second to first coefficient in power series
C expansion at the geocentre.
        CALL REL(ERRMAX,YSCAL,YIC,N)
C Set initial step size so that fifth order method would have relative error
C bound EPS.
        H=(EPS/ERRMAX)**(0.2D0)
C Find solutions and derivatives at minimum radius RMIN by power series
C expansions.
        XV=RMIN
        XVS=XV*XV
        XV3=XV*XVS
        XV4=XVS*XVS
        DO 44 I=1,6
          DO 45 J=1,3
C Find solutions.
            Y(I,J)=Y(I,J)+XVS*YSCAL(I,J)+XV4*YSCAL2(I,J)
C Find derivatives.
            YP(I,J)=2.D0*XV*YSCAL(I,J)+4.D0*XV3*YSCAL2(I,J)
 45       CONTINUE  
 44     CONTINUE
C Set Inner Core index.
        IR=1
 46     CONTINUE
C Calculate derivatives at beginning of current step.
        CALL YPRIME(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,
     1  IR,RMIN,ANGS,COR,WES)
C Initialize function values at beginning of integration step.
        DO 47 I=1,6
          DO 48 J=1,6
            Y1(I,J)=Y(I,J)
            Y2(I,J)=Y(I,J)
 48       CONTINUE  
 47     CONTINUE
C Integrate solution forward one full step. 
        CALL RK4(XV,Y1,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,H,IR,RMIN,
     1  ANGS,COR,WES)
C Integrate solution forward two half steps.
C Halve step.
        HH=0.5D0*H
C Reset radius.
        XV=XV-H
        CALL RK4(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,HH,IR,
     1  RMIN,ANGS,COR,WES)
        CALL YPRIME(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,IR,
     1  RMIN,ANGS,COR,WES)
        CALL RK4(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,HH,IR,
     1  RMIN,ANGS,COR,WES)
C Find error matrix and maximum relative error.
        DO 49 I=1,6
          DO 50 J=1,3
            DELTA(I,J)=Y1(I,J)-Y2(I,J)
 50       CONTINUE  
 49     CONTINUE
C Copy Y2(6,6) into Inner Core solution array YIC(6,3).
        DO 51 I=1,6
          DO 52 J=1,3
            YIC(I,J)=Y2(I,J)
 52       CONTINUE  
 51     CONTINUE    
C Find maximum relative error.
        CALL REL(ERRMAX,DELTA,YIC,N)
C Scale maximum relative error by specified relative error tolerence.
        ERRMAX=ERRMAX/EPS
C Test if step was successful.
        IF(ERRMAX.LT.1.D0) GO TO 53
C Step too large, reduce step size and repeat.
        XV=XV-H
        H=0.9D0*H*(ERRMAX**(-0.25D0))
        GO TO 46
 53     CONTINUE
C Step successful, accept solution and improve truncation error.
        DO 54 J=1,6
          DO 55 K=1,3
            Y(J,K)=Y2(J,K)+DELTA(J,K)/15.D0
 55       CONTINUE
 54     CONTINUE
C Increase step size.
        H=0.9D0*H*(ERRMAX**(-0.20D0))
C Test if solution is finished.
        XVT=XV+H
        IF(XVT.LT.R(N1)) GO TO 46
C Find remaining distance to the ICB.
        H=R(N1)-XV
C Compute fraction of radius.
        RATIO=H/R(N1)
C Continue integration only if remaining distance is more than
C 1.D-9 of radius.
        IF(RATIO.GT.1.D-9) GO TO 46
C Convert solutions back to y-variables.
C Set value of alpha for first two solutions.
        ANM2=AN-2.D0
C Convert three solutions.
        DO 56 J=1,3
C Set value of alpha for third solution.
        IF(J.EQ.3) ANM2=AN
C Find constant equal to the radius of the ICB to the power alpha.
        CONST=RIOB**ANM2
        Y(2,J)=CONST*Y(2,J)
        Y(4,J)=CONST*Y(4,J)
        CONST=CONST*RIOB
        Y(1,J)=CONST*Y(1,J)
        Y(3,J)=CONST*Y(3,J)
        Y(6,J)=CONST*Y(6,J)
        CONST=CONST*RIOB
        Y(5,J)=CONST*Y(5,J)
 56   CONTINUE
C Test if N=0.
        IF(N.EQ.0) GO TO 57
C Begin construction of coefficient matrix.
        DO 58 I=1,3
          CMC1(1,I)=Y(4,I)
          CMC1(2,I)=GZEROC*Y(1,I)-Y(2,I)/RHOAC-Y(5,I)
          CMC1(3,I)=4.D0*PI*G*RHOAC*Y(1,I)+Y(6,I)
 58     CONTINUE
C Set coefficient matrices equal.
        DO 59 I=1,3
          DO 60 J=1,3
            CMC2(I,J)=CMC1(I,J)
            CMC3(I,J)=CMC2(I,J)
 60       CONTINUE  
 59     CONTINUE
C Set constant vectors.
        BVC1(1)=0.D0
        BVC1(2)=1.D0
        BVC1(3)=0.D0
        BVC2(1)=0.D0
        BVC2(2)=0.D0
        BVC2(3)=-1.D0
C Store first constant vector.
        DO 61 I=1,3
          BVC3(I)=BVC1(I)
 61     CONTINUE
C Solve for linear combination coefficients.
        CALL LINSOL(CMC1,BVC1,3,CAUGC,DET,3,7)
        CALL LINSOL(CMC2,BVC2,3,CAUGC,DET,3,7)
C Alter coefficient matrix to apply subseismic condition.
        DO 62 I=1,3
          CMC3(3,I)=GZEROC*Y(1,I)-Y(2,I)/RHOAC
 62     CONTINUE  
C Solve for subseismic coefficients.
        CALL LINSOL(CMC3,BVC3,3,CAUGC,DET,3,7)
C Calculate unscaled Love numbers.
        SUM1=0.D0
        SUM2=0.D0
        SUM3=0.D0
        DO 63 I=1,3
          SUM1=SUM1+BVC1(I)*Y(1,I)
          SUM2=SUM2+BVC2(I)*Y(1,I)
          SUM3=SUM3+BVC3(I)*Y(1,I)
 63     CONTINUE
C Calculate radial derivative of V1n at ICB.
        SUM=0.D0
        DO 64 I=1,3
          SUM=SUM+(4.D0*PI*G*RHOAC*Y(1,I)+Y(6,I))*BVC3(I)
 64     CONTINUE
C Calculate potential condition parameter AND.
        AND=-AN/SUM
        GO TO 65
C Construct coefficient matrix for N=0.
 57     CONTINUE
        CMC1(1,1)=GZEROC*Y(1,3)-Y(2,3)/RHOAC-Y(5,3)
        CMC1(1,2)=1.D0
        CMC1(2,1)=4.D0*PI*G*RHOAC*Y(1,3)
        CMC1(2,2)=0.D0
C Solve for linear combination coefficients and AND.
        BVC1(1)=0.D0
        BVC2(1)=-1.D0/CMC1(2,1)
        AND=0.D0
C Calculate unscaled Love numbers.
        SUM1=BVC1(1)*Y(1,3)
        SUM2=BVC2(1)*Y(1,3)
        SUM3=0.D0
 65     CONTINUE
C Scale Love numbers to dimensionless values.
        BVC1(1)=GZEROC*SUM1
        BVC2(1)=GZEROC*SUM2/RIOB
        BVC3(1)=GZEROC*SUM3
C Test IF N=1.
        IF(N.NE.1) GO TO 66
        IF(ICOUNT.EQ.3) GO TO 67
        IF(ICOUNT.EQ.2) GO TO 68
        GAMA=1.D0/BVC3(1)
        COR=CORS
        ICOUNT=2
        GO TO 28
 68     CONTINUE
        BETA=(1.D0/BVC3(1)-GAMA)/SIG
        ANGS=ANGSS
        ICOUNT=3
        GO TO 28
 67     CONTINUE
        ALPHA=(1.D0/BVC3(1)-BETA*SIG-GAMA)/SIGS
C Write out internal Love numbers.
 66     CONTINUE
        WRITE(6,69)N,BVC1(1),BVC2(1)
 69     FORMAT(1X,/,I15,11X,F16.8,F28.8)
        WRITE(2,70)BVC3(1),AND
 70     FORMAT(2D23.15)
C End loop over degree for Inner Core.  
 26   CONTINUE
C Begin loop over degree for the Shell (Mantle + Crust).
      WRITE(6,25)
      DO 71 NP1=1,NN
        N=NP1-1
C Set up interpolation for the Mantle.
        N1=NM(3)
        N2=2*N1-2
        N3=N1-2
C Put Mantle values in active locations.
        DO 72 I=1,N1
          J=NK(3)+I
          R(I)=RI(J)
          RHO(I)=RHOI(J)
          MU(I)=MUI(J)
          LAMBDA(I)=LAMBDAI(J)
          GZERO(I)=GZEROI(J)
 72     CONTINUE
C Prepare for spline interpolation.
        CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Set initial values for fundamental solutions.
        DO 73 I=1,6
          Y(I,I)=1.D0
          DO 74 J=1,6
            IF(I.NE.J) Y(I,J)=0.D0
 74       CONTINUE
 73     CONTINUE
C Begin Runge-Kutta integration for the Mantle.
        M=NI(3)
        AM=DFLOAT(M)
C Find step size.
        H=(R(N1)-R(1))/AM
C Set initial radius.
        XV=R(1)
C Set integration region counter.
        IR=3
        DO 75 I=1,M
          CALL YPRIME(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,
     1    IR,RMIN,ANGS,COR,WES)
          CALL RK4(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,H,IR,
     1    RMIN,ANGS,COR,WES)
 75     CONTINUE
C Set up interpolation for the Crust.
        N1=NM(4)
        N2=2*N1-2
        N3=N1-2
C Put Crustal values in active locations.
        DO 76 I=1,N1
          J=NK(4)+I
          R(I)=RI(J)
          RHO(I)=RHOI(J)
          MU(I)=MUI(J)
          LAMBDA(I)=LAMBDAI(J)
          GZERO(I)=GZEROI(J)
 76     CONTINUE
C Prepare for spline interpolation.
        CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Begin Runge-Kutta integration for the Crust.
        M=NI(4)
        AM=DFLOAT(M)
C Find step size.
        H=(R(N1)-R(1))/AM
C Set initial radius.
        XV=R(1)
C Set integration region counter.
        IR=4
        DO 77 I=1,M
          CALL YPRIME(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,
     1    IR,RMIN,ANGS,COR,WES)
          CALL RK4(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,H,IR,
     1    RMIN,ANGS,COR,WES)
 77     CONTINUE
C Begin construction of coefficient matrix.
        AN=DFLOAT(N)
C Test if N=0.
        IF(N.EQ.0) GO TO 78
        DO 79 I=1,5
          J=I
          IF(I.GT.3)J=J+1
          CM1(1,I)=Y(2,J)
          CM1(2,I)=Y(4,J)
          CM1(3,I)=Y(5,J)+D*Y(6,J)/(AN+1.D0)
          CM1(4,I)=0.D0
          CM1(5,I)=0.D0
 79     CONTINUE
        CM1(4,1)=GZEROM
        CM1(4,2)=-1.D0/RHOBM
        CM1(4,4)=-1.D0
        CM1(5,1)=4.D0*PI*G*RHOBM
        CM1(5,2)=0.D0
        CM1(5,5)=1.D0
C Set coefficient matrices equal.
        DO 80 I=1,5
          DO 81 J=1,5
            CM2(I,J)=CM1(I,J)
            CM3(I,J)=CM2(I,J)
 81       CONTINUE  
 80     CONTINUE
C Set values of constant vectors.
        BV1(1)=0.D0
        BV1(2)=0.D0
        BV1(3)=0.D0
        BV1(4)=1.D0
        BV1(5)=0.D0
        DO 82 I=1,4
          BV2(I)=0.D0
 82     CONTINUE
        BV2(5)=-1.D0
C Store first constant vector.
        DO 83 I=1,5
          BV3(I)=BV1(I)
 83     CONTINUE
C Solve for linear combination coefficients.
        CALL LINSOL(CM1,BV1,5,CAUGS,DET,5,11)
        CALL LINSOL(CM2,BV2,5,CAUGS,DET,5,11)
C Alter coefficient matrix to apply subseismic condition.
        CM3(5,1)=GZEROM
        CM3(5,2)=-1.D0/RHOBM
        CM3(5,5)=0.D0
C Solve for subseismic coefficients.
        CALL LINSOL(CM3,BV3,5,CAUGS,DET,5,11)
C Calculate potential condition parameter BN.
        BN=(AN+1.D0)/(4.D0*PI*G*RHOBM*BV3(1)+BV3(5))
        GO TO 84
C Construct coefficient matrix for N=0.
C Reduce Y-matrix to 4x4.          
 78     CONTINUE
        DO 85 I=1,4
          K=I
          IF(K.GT.2) K=K+2
          DO 86 J=1,4
            L=J
            IF(L.GT.2) L=L+2
            Y(I,J)=Y(K,L)
 86       CONTINUE  
 85     CONTINUE
        DO 87 I=1,4
          CM01(1,I)=Y(2,I)
          CM01(2,I)=Y(3,I)+D*Y(4,I)/(AN+1.D0)
          CM01(3,I)=0.D0
          CM01(4,I)=0.D0
 87     CONTINUE
          CM01(3,1)=GZEROM
          CM01(3,2)=-1.D0/RHOBM
          CM01(3,3)=-1.D0
          CM01(4,1)=4.D0*PI*G*RHOBM
          CM01(4,2)=0.D0
          CM01(4,4)=1.D0
C Set coefficient matrices equal.
        DO 88 I=1,4
          DO 89 J=1,4
            CM02(I,J)=CM01(I,J)
            CM03(I,J)=CM02(I,J)
 89       CONTINUE  
 88     CONTINUE
C Set values of constant vectors.
        BV01(1)=0.D0
        BV01(2)=0.D0
        BV01(3)=1.D0
        BV01(4)=0.D0
        DO 90 I=1,3
           BV02(I)=0.D0
 90     CONTINUE
        BV02(4)=-1.D0
C Store first constant vector.
        DO 91 I=1,4
          BV03(I)=BV01(I)
 91     CONTINUE
C Solve for linear combination coefficients.
        CALL LINSOL(CM01,BV01,4,CAUGS0,DET,4,9)
        BV1(1)=BV01(1)
        CALL LINSOL(CM02,BV02,4,CAUGS0,DET,4,9)
        BV2(1)=BV02(1)
C Alter coefficient matrix to apply subseismic condition.
        CM03(4,1)=GZEROM
        CM03(4,2)=-1.D0/RHOBM
        CM03(4,4)=0.D0
C Solve for subseismic coefficients.
        CALL LINSOL(CM03,BV03,4,CAUGS0,DET,4,9)
        BV3(1)=BV03(1)
        BV3(4)=BV03(4) 
C Calculate potential condition parameter.
        BN=(AN+1.D0)/(4.D0*PI*G*RHOBM*BV3(1)+BV3(4))
C Scale Love numbers to dimensionless values.
 84    CONTINUE
        BV1(1)=GZEROM*BV1(1)
        BV2(1)=GZEROM*BV2(1)/RCMB
        BV3(1)=GZEROM*BV3(1)
C Write out internal Love numbers.
        WRITE(6,69)N,BV1(1),BV2(1)
        WRITE(2,70)BV3(1),BN
C End loop over degree.
 71   CONTINUE
C Write out dimensionless angular frequency coefficients of degree one
C reciprocal Love number.
      WRITE(2,92)ALPHA,BETA,GAMA
 92   FORMAT(3D23.15)
      END
C
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1        +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1        -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1         -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is the
C coefficient matrix, from subroutine SPMAT, relating the
C second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1 +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2 +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION P1(AN,LAMBDA,MU)
C Finds value of function P1 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      P1=2.D0*AN*(AN*(AN+2.D0)*LAMBDA+(AN*(AN+2.D0)-1.D0)*MU)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION P2(AN,LAMBDA,MU)
C Finds value of function P2 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      P2=AN*(AN+5.D0+(AN+3.D0)*(LAMBDA/MU))
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION Q1(AN,LAMBDA,MU)
C Finds value of function Q1 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      Q1=2.D0*AN*((AN+2.D0)*LAMBDA+(AN+1.D0)*MU)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION Q2(AN,LAMBDA,MU)
C Finds value of function Q2 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      Q2=2.D0*(AN+1.D0)+(AN+3.D0)*(LAMBDA/MU)
      RETURN
      END 
C
      SUBROUTINE MATRIX(A,RHO,LAMBDA,MU,PI,G,AN,WE)
C This subroutine calculates the 6X6 matrix A required to find higher
C terms in the power series expansions of the thee fundamental solutions
C at the geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(6,6),RHO(100)
      DOUBLE PRECISION LAMBDA(100),MU(100)
      BETA=1.D0/(LAMBDA(1)+2.D0*MU(1))
      DELTA=2.D0*MU(1)*(3.D0*LAMBDA(1)+2.D0*MU(1))*BETA
      GAMMA=(4.D0*PI*G*RHO(1)-2.D0*WE*WE)/3.D0
      AN1=AN*(AN+1.D0)
      EPSLN=4.D0*AN1*MU(1)*(LAMBDA(1)+MU(1))*BETA-2.D0*MU(1)
      A(1,1)=AN+3.D0+2.D0*LAMBDA(1)*BETA
      A(1,2)=-BETA
      A(1,3)=-AN1*LAMBDA(1)*BETA
      A(1,4)=0.D0
      A(1,5)=0.D0
      A(1,6)=0.D0
      A(2,1)=-2.D0*DELTA
      A(2,2)=AN+2.D0+4.D0*MU(1)*BETA
      A(2,3)=AN1*DELTA
      A(2,4)=-AN1
      A(2,5)=0.D0
      A(2,6)=0.D0
      A(3,1)=1.D0
      A(3,2)=0.D0
      A(3,3)=AN+2.D0
      A(3,4)=-1.D0/MU(1)
      A(3,5)=0.D0
      A(3,6)=0.D0
      A(4,1)=DELTA
      A(4,2)=LAMBDA(1)*BETA
      A(4,3)=-EPSLN
      A(4,4)=AN+5.D0
      A(4,5)=0.D0
      A(4,6)=0.D0
      A(5,1)=-3.D0*GAMMA-2.D0*WE*WE
      A(5,2)=0.D0
      A(5,3)=0.D0
      A(5,4)=0.D0
      A(5,5)=AN+4.D0
      A(5,6)=-1.D0
      A(6,1)=0.D0
      A(6,2)=0.D0
      A(6,3)=AN1*(3.D0*GAMMA+2.D0*WE*WE)
      A(6,4)=0.D0
      A(6,5)=-AN1
      A(6,6)=AN+5.D0
      RETURN
      END  
C
      SUBROUTINE LINSOL(A,B,N,C,DET,M1,M2)
C
C LINSOL solves the linear system Ax=b by Gauss-Jordan
C elimination with partial pivoting. A is the N x N 
C coefficient matrix and B is the N-length constant vector b.
C The inverse of A is returned in A, the solution vector x
C is returned in B, and the determinant of A is returned
C in DET. C is the N x 2N+1 augmented matrix used
C internally. M1 is the maximum dimension for N specified
C in the main programme and M2=2M1+1 is the maximum dimension
C for 2N+1 used in the main programme.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(M1,M1),X(M1),B(M1),C(M1,M2),HOLD(M1,M2),
     1IR(M1),IC(M1),IDET(M1)
C Construct the augmented matrix C.
      N2=2*N+1
      NP1=N+1
      NM1=N-1
C Insert the matrix A.
      DO 10 I=1,N
        DO 10 J=1,N
          C(I,J)=A(I,J)
 10     CONTINUE
C Insert the unknown vector.
      DO 11 J=1,N
        C(J,NP1)=B(J)
 11   CONTINUE
C Augment with the identity matrix.
      DO 12 I=1,N
        DO 12 J=1,N
        JJ=J+NP1
          C(I,JJ)=0.D0
            IF(I.EQ.J)C(I,JJ)=1.D0
 12   CONTINUE
c      write(9,200)((c(i,j),j=1,n2),i=1,n)
c 200  format(1x,7d11.4)
C Begin Gauss-Jordan elimination.
      DET=1.D0
      DO 13 I=1,N
        IM1=I-1
C Search for largest pivot.
        PIVOT=0.D0
        DO 14 J=1,N
          DO 14 K=1,N
C Scan IR and IC vectors for invalid pivot subscripts.
            IF(I.EQ.1)GO TO 15
            DO 16 ISC=1,IM1
              DO 16 JSC=1,IM1
              IF(J.EQ.IR(ISC))GO TO 14
              IF(K.EQ.IC(JSC))GO TO 14
 16           CONTINUE
 15           IF(DABS(C(J,K)).LE.DABS(PIVOT))GO TO 14
              PIVOT=C(J,K)
              IR(I)=J
              IC(I)=K
 14       CONTINUE
          DET=DET*PIVOT
C Label next row.
          IRP1=IR(I)+1
C Label previous row.
          IRM1=IR(I)-1
C Divide row by pivot.
          DO 17 J=1,N2
            C(IR(I),J)=C(IR(I),J)/PIVOT
 17       CONTINUE
C Reduce elements below pivot to zero.
          IF(IRP1.GT.N)GO TO 18
          DO 19 K=IRP1,N
            ELL=C(K,IC(I))
            DO 20 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 20         CONTINUE
 19       CONTINUE
 18       CONTINUE
C Reduce elements above pivot to zero.
          IF(IRM1.LT.1)GO TO 21
          DO 22 K=1,IRM1
            ELL=C(K,IC(I))
            DO 23 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 23         CONTINUE
 22       CONTINUE
 21       CONTINUE
 13   CONTINUE     
C Unscramble equations and record row interchanges.
      DO 24 I=1,N
        IDET(IR(I))=IC(I)
        DO 24 J=1,N2
        HOLD(IC(I),J)=C(IR(I),J)
 24   CONTINUE
C Put equations back in original matrix.
      DO 25 I=1,N
        DO 25 J=1,N2
        C(I,J)=HOLD(I,J)
 25   CONTINUE
C Determine sign of determinant.
      ICH=0
      DO 26 I=1,NM1
      IP1=I+1
        DO 26 J=IP1,N
        IF(IDET(J).GE.IDET(I))GO TO 26
        IHOLD=IDET(J)
        IDET(J)=IDET(I)
        IDET(I)=IHOLD
        ICH=ICH+1
 26   CONTINUE
      IF((ICH/2)*2.NE.ICH)DET=-DET
C Put inverse in A and solution in B.
      DO 27 I=1,N
      B(I)=C(I,NP1)
        DO 27 J=1,N
        JJ=J+NP1
        A(I,J)=C(I,JJ)
 27   CONTINUE
      RETURN
      END
C
      SUBROUTINE YPRIME(R,Y,A,YP,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,
     1IR,RMIN,ANGS,COR,WES)
C This subroutine finds the derivatives, YP(6,6), of the six fundamental
C solutions represented by the propagator matrix, Y(6,6), in the Mantle and
C Crust, at an arbitrary radius R for degree N>0. For N=0, the third and
C fourth solution vectors can be ignored.
C In the Inner Core, it finds the derivatives of the fundamental solutions
C in terms of the z-variables which are transformations of the y-variables. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RM(100),RHOM(100),GZEROM(100),C(100,100),
     1Y(6,6),YP(6,6),A(6,6)
      DOUBLE PRECISION MUM(100),LAMBDAM(100),MU,LAMBDA 
C Zero fill derivative matrix YP(I,J) and coefficient matrix A(I,J).
      DO 10 I=1,6
        DO 11 J=1,6
          YP(I,J)=0.D0
          A(I,J)=0.D0
 11     CONTINUE
 10   CONTINUE
C Set maximum number of points for interpolation.
      M1=100
C If R is less than or equal to RMIN, take properties to be those at the
C geocentre.
      IF(R.LE.RMIN) GO TO 12
C Interpolate Earth properties at radius R.
      CALL INTPL(R,RHO,N1,C,RM,RHOM,M1)
      CALL INTPL(R,MU,N1,C,RM,MUM,M1)
      CALL INTPL(R,LAMBDA,N1,C,RM,LAMBDAM,M1)         
      CALL INTPL(R,GZERO,N1,C,RM,GZEROM,M1)
      GO TO 13
C Assign geocentric Earth properties.
 12   RHO=RHOM(1)
      MU=MUM(1)
      LAMBDA=LAMBDAM(1)
      GZERO=(4.D0*PI*G*RHO-2.D0*WES)*R/3.D0
C Construct matrix A.
 13   AN=DFLOAT(N)
C Define factors.
      F1=AN*(AN+1.D0)
      F2=LAMBDA+2.D0*MU
      F3=3.D0*LAMBDA+2.D0*MU
      F4=RHO*GZERO
      F5=4.D0*PI*G*RHO
C Assign values to matrix elements.
      A(1,1)=-2.D0*LAMBDA/(F2*R)
      A(1,2)=1.D0/F2
      A(1,3)=F1*LAMBDA/(F2*R)
      A(1,4)=0.D0
      A(1,5)=0.D0
      A(1,6)=0.D0
      A(2,1)=4.D0*(-F4+MU*F3/(F2*R))/R-2.D0*RHO*WES-ANGS*RHO
      A(2,2)=-4.D0*MU/(F2*R)
      A(2,3)=F1*(F4-2.D0*MU*F3/(F2*R))/R+COR*RHO
      A(2,4)=F1/R
      A(2,5)=0.D0
      A(2,6)=-RHO
      IF(N.EQ.0.AND.IR.EQ.1) GO TO 14
      A(3,1)=-1.D0/R
      A(3,2)=0.D0
      A(3,3)=1.D0/R
      A(3,4)=1.D0/MU
      A(3,5)=0.D0
      A(3,6)=0.D0
      A(4,1)=(F4-2.D0*MU*F3/(F2*R))/R
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,1)=A(4,1)+COR*RHO/F1
      A(4,2)=-LAMBDA/(F2*R)
      A(4,3)=2.D0*MU*((2.D0*F1-1.D0)*LAMBDA+(2.D0*F1-2.D0)*MU)/(F2*R*R)
     1-ANGS*RHO
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,3)=A(4,3)+COR*RHO/F1
      A(4,4)=-3.D0/R
      A(4,5)=-RHO/R
      A(4,6)=0.D0
 14   CONTINUE
      A(5,1)=F5
      A(5,2)=0.D0
      A(5,3)=0.D0
      A(5,4)=0.D0
      A(5,5)=0.D0
      A(5,6)=1.D0
      IF (N.EQ.0.AND.IR.EQ.1) GO TO 15
      A(6,1)=0.D0
      A(6,2)=0.D0
      A(6,3)=-F5*F1/R
      A(6,4)=0.D0
      A(6,5)=F1/(R*R)
 15   CONTINUE
      A(6,6)=-2.D0/R
C Branch to Inner Core calculation
      IF(IR.EQ.1) GO TO 16
C Multiply A into Y to find derivative matrix for Shell.
      DO 17 I=1,6
        DO 18 J=1,6
          DO 19 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 19       CONTINUE
 18     CONTINUE    
 17   CONTINUE
      GO TO 20
C Modify matrix A for Inner Core calculation.
 16   A(1,2)=A(1,2)/R
      A(2,1)=A(2,1)*R
      A(2,3)=A(2,3)*R
      A(2,6)=A(2,6)*R
      A(3,4)=A(3,4)/R
      A(4,1)=A(4,1)*R
      A(4,3)=A(4,3)*R
      A(4,5)=A(4,5)*R*R
      A(5,1)=A(5,1)/R
      A(5,6)=A(5,6)/R
      A(6,5)=A(6,5)*R
C Multiply A into Y to find YP for Inner Core.
      DO 21 I=1,6
        DO 22 J=1,3
          DO 23 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 23       CONTINUE
 22     CONTINUE    
 21   CONTINUE
C Correct YP to give derivatives of z-variables.
C Set value of alpha plus one for first two solutions.
      ANM1=AN-1.D0
      DO 24 J=1,3
C Set value of alpha plus one for third solution
        IF(J.EQ.3)ANM1=AN+1.D0
        YP(1,J)=YP(1,J)-ANM1*Y(1,J)/R
        YP(2,J)=YP(2,J)-(ANM1-1.D0)*Y(2,J)/R
        YP(3,J)=YP(3,J)-ANM1*Y(3,J)/R
        YP(4,J)=YP(4,J)-(ANM1-1.D0)*Y(4,J)/R
        YP(5,J)=YP(5,J)-(ANM1+1.D0)*Y(5,J)/R
        YP(6,J)=YP(6,J)-ANM1*Y(6,J)/R
 24   CONTINUE
 20   CONTINUE
      RETURN
      END
C
      SUBROUTINE REL(ERRMAX,DELTA,YIC,N)
C This subroutine is used in the variable step size Runge-Kutta integration
C of the fundamental solutions in the Inner Core. It finds the maximum
C relative error in the elements of the difference array, DELTA(6,3), compared
C to the elements of the fundamental solution array, YIC(6,3),for degree N.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DELTA(6,3),YIC(6,3)
C Zero relative error.
      ERRMAX=0.D0
      DO 1 I=1,6
C Branch for degee zero.
        IF(N.EQ.0)GO TO 2
        GO TO 3
C If degree is zero skip y_3 and y_4.
 2      IF(I.EQ.3.OR.I.EQ.4)GO TO 1
 3      DO 4 J=1,3
C Third solution is only fundamental solution for degree zero.
        IF(N.EQ.0.AND.J.NE.3)GO TO 4
        IF(YIC(I,J).EQ.0.D0)GO TO 4
        RATIO=DABS(DELTA(I,J)/YIC(I,J))
        ERRMAX=DMAX1(ERRMAX,RATIO)
 4      CONTINUE
 1    CONTINUE
      RETURN
      END  
C
      SUBROUTINE RK4(R,Y,A,K1,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,
     1PI,G,H,IR,RMIN,ANGS,COR,WES)
C This subroutine completes the last three steps of a fourth order
C Runge-Kutta integration, given the derivatives K1(6,6) at the starting
C radius R.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RM(100),RHOM(100),GZEROM(100),C(100,100),
     1Y(6,6),YY(6,6),A(6,6)
      DOUBLE PRECISION MUM(100),LAMBDAM(100),MU,LAMBDA,
     1K1(6,6),K2(6,6),K3(6,6),K4(6,6)
C Set half step size, one sixth step size.
      HH=0.5*H
      H6=H/6.D0
C Set number of solutions for shell.
      NS=6
C If region is Inner Core, reset number of solutions to three.
      IF(IR.EQ.1)NS=3
C Complete remaining steps of Runge-Kutta integration.
      DO 1 I=1,6
        DO 1 J=1,NS
          YY(I,J)=Y(I,J)+HH*K1(I,J)
 1    CONTINUE
C Increment radius and solve for new derivatives.
      R=R+HH    
      CALL YPRIME(R,YY,A,K2,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1RMIN,ANGS,COR,WES)
      DO 2 I=1,6
        DO 2 J=1,NS
          YY(I,J)=Y(I,J)+HH*K2(I,J)
 2    CONTINUE
C Find new derivatives.
      CALL YPRIME(R,YY,A,K3,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1RMIN,ANGS,COR,WES)
      DO 3 I=1,6
        DO 3 J=1,NS
          YY(I,J)=Y(I,J)+H*K3(I,J)
 3    CONTINUE
C Increment radius and solve for new derivatives.    
      R=R+HH
      CALL YPRIME(R,YY,A,K4,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1RMIN,ANGS,COR,WES)
      DO 4 I=1,6
        DO 4 J=1,NS
          Y(I,J)=Y(I,J)+H6*(K1(I,J)+2.D0*(K2(I,J)+K3(I,J))+K4(I,J))
 4    CONTINUE
      RETURN
      END    
              
